import React, { Component, Fragment } from 'react';
import { queryConfig } from '../services';
import { getInfo } from '@/highOrderComponent/Service';
import { message, Button } from 'antd';
import Shell from '@/baseComponent/Shell';
import ButtonDiy from '@/baseComponent/ButtonDiy';
import router from 'umi/router';
import DetailDom from './DetailDom';
import { A4Width, A4Height, getCLodopFuncJS, templateCode } from './config';
import RenderAuthorized from '@/components/Authorized';
import { getAuthority } from '@/utils/authority';
import { ModalInfo } from '@/baseComponent/Modal';

const Authorized = RenderAuthorized(getAuthority());

export default class ViewPrint extends Component {
	constructor(props) {
		super(props);
		this.state = {
			configAll: null,
			viewData: null,
			loading: true,
			printIndex: 0,
			showWindowPrint: false,
		};
		this.getConfigInfo = queryConfig.bind(this);
	}

	getViewData = () => {
		const { selectedRows } = this.props;
		const ids = selectedRows.map((x) => x.id);
		this.getConfigInfo().then((x) => {
			if (x && x.queryUrl) {
				getInfo(
					{
						ids: ids.join(','),
					},
					x.queryUrl,
				).then((viewData) => {
					if (!viewData || !viewData.length) {
						message.warning('未查询到可打印的奖状数据');
						console.error(`${x.queryUrl}接口报错或者没有返回数据`);
						return false;
					}
					let i = 0;
					for (const item of viewData) {
					//	item.name = ['黎处', '王昊楠', '古力娜扎', '古丽尼帕尔·阿布都黑利力'][i] || '钟是志';
						i++;
						for (let z in item) {
							if (item[z] === null || item[z] === 'null') {
								item[z] = '';
							}
						}
					}
					this.setState({
						viewData,
					});
				});
			}
		});
	};

	componentDidMount() {
		this.getViewData();
		getCLodopFuncJS().then((response) => {
			window.On_CLodop_Opened = function() {
				console.log('打印插件准备就绪,随时可以打印');
				window.On_CLodop_Opened = null;
			};
			if (response) {
				this.setState({
					loading: false,
				});
			}
		});
	}

	detailDom = (data) => {
		const {
			configAll: { config },
		} = this.state;
		const res = [];
		for (const item of config) {
			res.push(
				DetailDom({
					config: item,
					data: data[item.fieldCode],
				}),
			);
		}
		return res;
	};

	printOne = (review = false) => {
		const { printIndex, configAll, viewData } = this.state;
		const { printMeth } = configAll;
		const { wide, high } = configAll;
		const { LODOP } = window;
		let strHTML = document.getElementById(`printDomId_${printIndex}`).innerHTML;
		if (review) {
			strHTML = document.getElementById(`printDomId_0`).innerHTML;
		}
		LODOP.PRINT_INITA(0, 0, `${wide}cm`, `${high}cm`, `printDomId_${printIndex}`);
		LODOP.SET_PRINT_PAGESIZE(printMeth, `${wide}cm`, `${high}cm`, 'CreateCustomPage'); // 打印方向
		LODOP.ADD_PRINT_HTM(0, 0, '100%', '100%', strHTML); // HTML 打印
		if (review) {
			LODOP.PREVIEW(); // 打印预览
		} else {
			LODOP.PRINT(); // 打印
		}
	};

	printPreview = () => {
		// 打印预览
		this.printOne(true);
	};

	printedDataSave = (index) => {
		const { selectedRows } = this.props;
		const { configAll } = this.state;
		if (!selectedRows[index] || !selectedRows[index].id) {
			console.error('printedDataSave 没有正确的id 无法保存打印信息到后台');
			return false;
		}
		const ids = selectedRows[index].id;
		getInfo({ ids }, configAll.callUrl).then((x) => {
			if (x) {
				console.log(`${ids}的打印信息已保存到后台`);
				return true;
			} else {
				return false;
			}
		});
	};

	printOneByOne = () => {
		// 按队列打印
		const { viewData, printIndex } = this.state;
		if (viewData && viewData.length) {
			this.printOne();
			const { length } = viewData;
			let LODOPObj = window.LODOP;
			LODOPObj.On_Return_Remain = true;
			LODOPObj.On_Return = (TaskId, Value) => {
				if (Number(Value) === 1) {
					message.info(`正在打印第${printIndex + 1}张`);
					this.printedDataSave(printIndex);
				}
				this.setState({ printIndex: this.state.printIndex + 1 }, () => {
					if (this.state.printIndex < length) {
						this.printOne();
					} else {
						this.setState({
							loading: false,
						});
					}
				});
			};
		} else {
			console.error('暂无任何数据无法打印');
		}
	};

	printByLodop = () => {
		if (!window.LODOP || !window.LODOP.PRINT_INIT) {
			console.error('打印服务未启动');
			ModalInfo('打印服务未启动');
			return false;
		}
		this.setState(
			{
				showWindowPrint: true,
				printIndex: 0,
			},
			() => {
				this.printOneByOne();
			},
		);
	};

	render() {
		const { configAll, viewData, showWindowPrint, loading } = this.state;
		if (!viewData) {
			return 	<Shell styleShell={{ marginTop: '0', marginBottom: '20px' }}>
        <div style={{ height: '54px', padding: '12px 0 12px 12px' }}>
          <ButtonDiy
            name={'返回'}
            handleClick={() => {
              router.goBack();
            }}
          />
        </div>
      </Shell>;
		}
		if (!configAll.backgroundUrl) {
			console.error('没有设置模版图片无法使用');
			return null;
		}
		const { wide, high } = configAll;
		const imageStyle = {
			height: high ? `${high}cm` : A4Height,
			width: wide ? `${wide}cm` : A4Width,
			textAlign: 'center',
		};
		const { pathname } = this.props.location;
		const templateInfo = templateCode.find((x) => {
			return x.path === pathname;
		});
		if(!templateInfo){
		  console.error('未找到templateInfo', pathname);
    }
		return (
			<Fragment>
				<Shell styleShell={{ marginTop: '0', marginBottom: '20px' }}>
					<div style={{ height: '54px', padding: '12px 0 12px 12px' }}>
						<ButtonDiy
							name={'打印设备选择'}
							handleClick={() => {
								window.LODOP.SELECT_PRINTER();
							}}
							loading={loading}
						/>
						<ButtonDiy
							name={'打印'}
							handleClick={() => {
								this.printByLodop();
							}}
							loading={loading}
						/>

						<ButtonDiy
							name={'返回'}
							handleClick={() => {
								router.goBack();
							}}
						/>

						<Authorized authority={'/jc/setting/PrintPositionSetting'}>
							<span style={{ float: 'right' }}>
								<ButtonDiy
									name={'打印预览(实施)'}
									handleClick={() => {
										this.printPreview();
									}}
									loading={loading}
								/>
							</span>
						</Authorized>
						<Authorized authority={'/jc/setting/PrintPositionSetting'}>
							<span style={{ float: 'right' }}>
								<ButtonDiy
									name={'打印模版设置(实施)'}
									handleClick={() => {
										router.push({
											pathname: '/jc/setting/PrintConfig',
											state: { code: templateInfo.code },
										});
									}}
								/>
							</span>
						</Authorized>

						<Authorized authority={'/jc/setting/PrintPositionSetting'}>
							<span style={{ float: 'right' }}>
								<ButtonDiy
									name={'字段位置设置(实施)'}
									handleClick={() => {
										router.push({
											pathname: '/xg/xg_hard/AwardSet/printSetting',
											state: { code: templateInfo.code },
										});
									}}
								/>
							</span>
						</Authorized>
					</div>
				</Shell>
				{viewData && viewData.length ? (
					<div
						ref={(node) => (this.content = node)}
						style={{
							display: showWindowPrint || true ? 'block' : 'none',
						}}>
						{viewData.map((info, index) => {
							return (
								<div key={`${index}divKey`} id={`printDomId_${index}`}>
									<div
										style={{
											position: 'relative',
											pageBreakAfter: 'always',
											marginBottom: '10px',
											...imageStyle,
											backgroundColor: '#fff',
										}}>
										{this.detailDom(info)}
									</div>
								</div>
							);
						})}
					</div>
				) : null}
			</Fragment>
		);
	}
}
